/*
  Licensed to the Court of the University of Edinburgh (UofE) under one
  or more contributor license agreements.  See the NOTICE file
  distributed with this work for additional information
  regarding copyright ownership.  The UofE licenses this file
  to you under the Apache License, Version 2.0 (the
  "License"); you may not use this file except in compliance
  with the License.  You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing,
  software distributed under the License is distributed on an
  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
  KIND, either express or implied.  See the License for the
  specific language governing permissions and limitations
  under the License.
*/

package uk.ac.ed.inf.graph.util.impl;

import java.util.Collection;
import java.util.Comparator;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

import uk.ac.ed.inf.graph.compound.ICompoundEdge;
import uk.ac.ed.inf.graph.compound.ICompoundNode;
import uk.ac.ed.inf.graph.util.INodeSet;

public final class NodeSet<
		N extends ICompoundNode,
		E extends ICompoundEdge
> implements INodeSet<N, E> {
	
	private final SortedSet<N> nodeList;

	public NodeSet(){
		this.nodeList = new TreeSet<N>();
	}

	@Override
	public boolean add(N o) {
		return nodeList.add(o);
	}

	@Override
	public boolean addAll(Collection<? extends N> c) {
		return nodeList.addAll(c);
	}

	@Override
	public void clear() {
		nodeList.clear();
	}

	public Comparator<? super N> comparator() {
		return nodeList.comparator();
	}

	@Override
	public boolean contains(int nodeIdx){
		return this.basicGet(nodeIdx) != null;
	}
	
	@Override
	public boolean contains(Object o) {
		return nodeList.contains(o);
	}

	N basicGet(int nodeIdx){
		N retVal = null;
		for(N node : this.nodeList){
			if(node.getIndex() == nodeIdx){
				retVal = node;
				break;
			}
		}
		return retVal;
	}
	
	@Override
	public N get(int nodeIdx){
		N retVal = basicGet(nodeIdx);
		if(retVal == null){
			throw new IllegalArgumentException("set does not contain a node at this position nodeIdx = " + nodeIdx);
		}
		return retVal;
	}
	
	@Override
	public boolean containsAll(Collection<?> c) {
		return nodeList.containsAll(c);
	}

	@Override
	public boolean equals(Object o) {
		return nodeList.equals(o);
	}

	@Override
	public int hashCode() {
		return nodeList.hashCode();
	}

	@Override
	public boolean isEmpty() {
		return nodeList.isEmpty();
	}

	@Override
	public Iterator<N> iterator() {
		return nodeList.iterator();
	}

	@Override
	public boolean remove(Object o) {
		return nodeList.remove(o);
	}

	@Override
	public boolean removeAll(Collection<?> c) {
		return nodeList.removeAll(c);
	}

	@Override
	public boolean retainAll(Collection<?> c) {
		return nodeList.retainAll(c);
	}

	@Override
	public int size() {
		return nodeList.size();
	}

	@Override
	public Object[] toArray() {
		return nodeList.toArray();
	}

	@Override
	public <T> T[] toArray(T[] a) {
		return nodeList.toArray(a);
	}
	
	@Override
	public String toString(){
		return this.nodeList.toString();
	}
}
